------------------------------------------------------------------------------
--                                                                          --
--                           GNAT libplot binding                           --
--                                                                          --
--                             Sierpinski Curves                            --
--                                                                          --
--                                 B O D Y                                  --
--                                                                          --
------------------------------------------------------------------------------
-- Copyright (c) 1998 by Juergen Pfeifer                                    --
--                                                                          --
-- Permission is hereby granted, free of charge, to any person obtaining a  --
-- copy of this software and associated documentation files (the            --
-- "Software"), to deal in the Software without restriction, including      --
-- without limitation the rights to use, copy, modify, merge, publish,      --
-- distribute, distribute with modifications, sublicense, and/or sell       --
-- copies of the Software, and to permit persons to whom the Software is    --
-- furnished to do so, subject to the following conditions:                 --
--                                                                          --
-- The above copyright notice and this permission notice shall be included  --
-- in all copies or substantial portions of the Software.                   --
--                                                                          --
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  --
-- OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               --
-- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   --
-- IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   --
-- DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    --
-- OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    --
-- THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               --
--                                                                          --
-- Except as contained in this notice, the name(s) of the above copyright   --
-- holders shall not be used in advertising or otherwise to promote the     --
-- sale, use or other dealings in this Software without prior written       --
-- authorization.                                                           --
------------------------------------------------------------------------------
--  Author: Juergen Pfeifer <Juergen.Pfeifer@T-Online.de> 1998
--  Version Control
--  $Revision: 1.2 $
------------------------------------------------------------------------------
--
--  This is a very simple sample program.

pragma Warnings (Off);
with GNU.plotutil.Motif;
pragma Warnings (On);

with GNU.plotutil.Device.Direct;
use  GNU.plotutil;
with GNU.plotutil.iplot;

procedure Sierpinski is
   --  This procedure plots the Sierpinski curves of order 1 .. N
   --  The algorithm is taken from
   --  N.Wirth: Algorithmen und Datenstrukturen, 2. Auflage 1979
   --  Translated from Pascal to Ada :-)
   --
   package X_Plotter is new GNU.plotutil.Device.Direct (X_Device_Type);
   package MyPlotter renames X_Plotter;

   type Coordinate is new Integer range 0 .. 512;
   package IP is new GNU.plotutil.iplot (Coordinate); use IP;

   N  : constant := 5;
   H0 : constant := Coordinate'Last;

   H  : Coordinate := H0 / 4;
   X0 : Coordinate := 2 * H;
   Y0 : Coordinate := 3 * H;

   X, Y : Coordinate;
   I : Natural := 0;

   procedure B (I : in Natural);
   procedure C (I : in Natural);
   procedure D (I : in Natural);

   procedure Plot is
   begin
      Continue (X, Y);
   end Plot;
   pragma Inline (Plot);

   procedure A (I : in Natural) is
   begin
      if I > 0 then
         A (I - 1); X := X + H; Y := Y - H; Plot;
         B (I - 1); X := X + 2 * H; Plot;
         D (I - 1); X := X + H; Y := Y + H; Plot;
         A (I - 1);
      end if;
   end A;
   pragma Inline (A);

   procedure B (I : in Natural) is
   begin
      if (I > 0) then
         B (I - 1); X := X - H; Y := Y - H; Plot;
         C (I - 1); Y := Y - 2 * H; Plot;
         A (I - 1); X := X + H; Y := Y - H; Plot;
         B (I - 1);
      end if;
   end B;
   pragma Inline (B);

   procedure C (I : in Natural) is
   begin
      if I > 0 then
         C (I - 1); X := X - H; Y := Y + H; Plot;
         D (I - 1); X := X - 2 * H; Plot;
         B (I - 1); X := X - H; Y := Y - H; Plot;
         C (I - 1);
      end if;
   end C;
   pragma Inline (C);

   procedure D (I : in Natural) is
   begin
      if I > 0 then
         D (I - 1); X := X + H; Y := Y + H; Plot;
         A (I - 1); Y := Y + 2 * H; Plot;
         C (I - 1); X := X - H; Y := Y + H; Plot;
         D (I - 1);
      end if;
   end D;
   pragma Inline (D);

   procedure Startplot is
   begin
      Select_Plotter (MyPlotter.Create);
      Open;
      Space (Coordinate'First, Coordinate'First,
             Coordinate'Last, Coordinate'Last);
      Background_ColorName ("yellow");
      Line_Width (1);
      Pen_ColorName ("peru");
      Erase;
   end Startplot;

   procedure Setplot is
   begin
      Move (X, Y);
   end Setplot;
   pragma Inline (Setplot);

   procedure Endplot is
   begin
      Close;
      Delete;
   end Endplot;
   pragma Inline (Endplot);

begin
   Startplot;

   loop
      I := I + 1;
      X0 := X0 - H;
      H := H / 2;
      Y0 := Y0 + H;
      X := X0; Y := Y0; Setplot;
      A (I); X := X + H; Y := Y - H; Plot;
      B (I); X := X - H; Y := Y - H; Plot;
      C (I); X := X - H; Y := Y + H; Plot;
      D (I); X := X + H; Y := Y + H; Plot;
      exit when I = N;
   end loop;

   Endplot;

end Sierpinski;
