------------------------------------------------------------------------------
--                                                                          --
--                           GNAT libplot binding                           --
--                                                                          --
--                            GNU.plotutil.cplot                            --
--                                                                          --
--                                 B O D Y                                  --
--                                                                          --
------------------------------------------------------------------------------
-- Copyright (c) 1998 by Juergen Pfeifer                                    --
--                                                                          --
-- Permission is hereby granted, free of charge, to any person obtaining a  --
-- copy of this software and associated documentation files (the            --
-- "Software"), to deal in the Software without restriction, including      --
-- without limitation the rights to use, copy, modify, merge, publish,      --
-- distribute, distribute with modifications, sublicense, and/or sell       --
-- copies of the Software, and to permit persons to whom the Software is    --
-- furnished to do so, subject to the following conditions:                 --
--                                                                          --
-- The above copyright notice and this permission notice shall be included  --
-- in all copies or substantial portions of the Software.                   --
--                                                                          --
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  --
-- OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               --
-- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   --
-- IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   --
-- DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    --
-- OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    --
-- THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               --
--                                                                          --
-- Except as contained in this notice, the name(s) of the above copyright   --
-- holders shall not be used in advertising or otherwise to promote the     --
-- sale, use or other dealings in this Software without prior written       --
-- authorization.                                                           --
------------------------------------------------------------------------------
--  Author: Juergen Pfeifer <Juergen.Pfeifer@T-Online.de> 1998
--  Version Control
--  $Revision: 1.2 $
------------------------------------------------------------------------------
with GNU.plotutil.fplot;

package body GNU.plotutil.cplot is

   package RP is new GNU.plotutil.fplot (Real); use RP;

   procedure Space (C0, C1 : in Complex) is
   begin
      Space (Re (C0), Im (C0), Re (C1), Im (C1));
   end Space;

   procedure Space_Extended (C0, C1, C2 : in Complex) is
   begin
      Space_Extended (Re (C0), Im (C0),
                      Re (C1), Im (C1),
                      Re (C2), Im (C2));
   end Space_Extended;

   procedure Move (C : in Complex) is
   begin
      Move (Re (C), Im (C));
   end Move;

   procedure Move_Relative (C : in Complex) is
   begin
      Move_Relative (Re (C), Im (C));
   end Move_Relative;

   procedure Continue (C : in Complex) is
   begin
      Continue (Re (C), Im (C));
   end Continue;

   procedure Continue_Relative (C : in Complex) is
   begin
      Continue_Relative (Re (C), Im (C));
   end Continue_Relative;

   procedure Line_Width (Size : in Real'Base) is
   begin
      RP.Line_Width (Size);
   end Line_Width;

   procedure Line_Width is
   begin
      RP.Line_Width;
   end Line_Width;

   procedure Line (C0, C1 : in Complex) is
   begin
      Line (Re (C0), Im (C0), Re (C1), Im (C1));
   end Line;

   procedure Line_Relative (C0, C1 : in Complex) is
   begin
      Line_Relative (Re (C0), Im (C0), Re (C1), Im (C1));
   end Line_Relative;

   procedure Point (C : in Complex) is
   begin
      Point (Re (C), Im (C));
   end Point;

   procedure Point_Relative (C : in Complex) is
   begin
      Point_Relative (Re (C), Im (C));
   end Point_Relative;

   procedure Arc (C, C0, C1 : in Complex) is
   begin
      Arc (Re (C), Im (C), Re (C0), Im (C0), Re (C1), Im (C1));
   end Arc;

   procedure Arc_Relative (C, C0, C1 : in Complex) is
   begin
      Arc_Relative (Re (C), Im (C), Re (C0), Im (C0), Re (C1), Im (C1));
   end Arc_Relative;

   procedure Elliptical_Arc (C, C0, C1 : in Complex) is
   begin
      Elliptical_Arc (Re (C), Im (C), Re (C0), Im (C0), Re (C1), Im (C1));
   end Elliptical_Arc;

   procedure Elliptical_Arc_Relative (C, C0, C1 : in Complex) is
   begin
      Elliptical_Arc_Relative (Re (C), Im (C),
                               Re (C0), Im (C0),
                               Re (C1), Im (C1));
   end Elliptical_Arc_Relative;

   procedure Box (C0, C1 : in Complex) is
   begin
      Box (Re (C0), Im (C0), Re (C1), Im (C1));
   end Box;

   procedure Box_Relative (C0, C1 : in Complex) is
   begin
      Box_Relative (Re (C0), Im (C0), Re (C1), Im (C1));
   end Box_Relative;

   procedure Circle (C : in Complex; R : in Real'Base) is
   begin
      Circle (Re (C), Im (C), R);
   end Circle;

   procedure Circle_Relative (C : in Complex; R : in Real'Base) is
   begin
      Circle_Relative (Re (C), Im (C), R);
   end Circle_Relative;

   procedure Ellipse (C : in Complex; Rx, Ry, Angle : in Real'Base) is
   begin
      Ellipse (Re (C), Im (C), Rx, Ry, Angle);
   end Ellipse;

   procedure Ellipse_Relative (C : in Complex; Rx, Ry, Angle : in Real'Base) is
   begin
      Ellipse_Relative (Re (C), Im (C), Rx, Ry, Angle);
   end Ellipse_Relative;

   procedure Quadratic_Bezier_Curve (C0, C1, C2 : in Complex) is
   begin
      Quadratic_Bezier_Curve (Re (C0), Im (C0),
                              Re (C1), Im (C1),
                              Re (C2), Im (C2));
   end Quadratic_Bezier_Curve;

   procedure Quadratic_Bezier_Curve_Relative (C0, C1, C2 : in Complex) is
   begin
      Quadratic_Bezier_Curve_Relative (Re (C0), Im (C0),
                                       Re (C1), Im (C1),
                                       Re (C2), Im (C2));
   end Quadratic_Bezier_Curve_Relative;

   procedure Cubic_Bezier_Curve (C0, C1, C2, C3 : in Complex) is
   begin
      Cubic_Bezier_Curve (Re (C0), Im (C0),
                          Re (C1), Im (C1),
                          Re (C2), Im (C2),
                          Re (C3), Im (C3));
   end Cubic_Bezier_Curve;

   procedure Cubic_Bezier_Curve_Relative (C0, C1, C2, C3 : in Complex) is
   begin
      Cubic_Bezier_Curve_Relative (Re (C0), Im (C0),
                                   Re (C1), Im (C1),
                                   Re (C2), Im (C2),
                                   Re (C3), Im (C3));
   end Cubic_Bezier_Curve_Relative;

   procedure Marker (C     : in Complex;
                     Shape : in Marker_Type;
                     Size  : in Real'Base) is
   begin
      Marker (Re (C), Im (C), Shape, Size);
   end Marker;

   procedure Marker (C    : in Complex;
                     Ch   : in Character;
                     Size : in Real'Base) is
   begin
      Marker (Re (C), Im (C), Ch, Size);
   end Marker;

   procedure Marker_Relative (C     : in Complex;
                              Shape : in Marker_Type;
                              Size  : in Real'Base) is
   begin
      Marker_Relative (Re (C), Im (C), Shape, Size);
   end Marker_Relative;

   procedure Marker_Relative (C    : in Complex;
                              Ch   : in Character;
                              Size : in Real'Base) is
   begin
      Marker_Relative (Re (C), Im (C), Ch, Size);
   end Marker_Relative;

   function Text_Angle (Angle : in Real'Base) return Real'Base is
   begin
      return RP.Text_Angle (Angle);
   end Text_Angle;

   function Font_Name (Name : in String) return Real'Base is
   begin
      return RP.Font_Name (Name);
   end Font_Name;

   function Font_Name return Real'Base is
   begin
      return RP.Font_Name;
   end Font_Name;

   function Font_Size (Size : in Real'Base) return Real'Base is
   begin
      return RP.Font_Size (Size);
   end Font_Size;

   function Font_Size return Real'Base is
   begin
      return RP.Font_Size;
   end Font_Size;

   function Label_Width (Text : in String) return Real'Base is
   begin
      return RP.Label_Width (Text);
   end Label_Width;

   procedure Concatenate_Transformation (M0, M1, M2, M3 : in Real'Base;
                                         T : in Complex) is
   begin
      Concatenate_Transformation (M0, M1, M2, M3, Re (T), Im (T));
   end Concatenate_Transformation;

   procedure Rotate (Theta : in Real'Base) is
   begin
      RP.Rotate (Theta);
   end Rotate;

   procedure Scale (Scale : in Complex) is
   begin
      RP.Scale (Re (Scale), Im (Scale));
   end Scale;

   procedure Translate (T : in Complex) is
   begin
      Translate (Re (T), Im (T));
   end Translate;

   procedure Line_Dash (Dash_Distances : in Dashes;
                        Offset         : in Real'Base) is
   begin
      RP.Line_Dash (RP.Dashes (Dash_Distances), Offset);
   end Line_Dash;

end GNU.plotutil.cplot;
