------------------------------------------------------------------------------
--                                                                          --
--                           GNAT libplot binding                           --
--                                                                          --
--                           GNU.plotutil.Device                            --
--                                                                          --
--                                 B O D Y                                  --
--                                                                          --
------------------------------------------------------------------------------
-- Copyright (c) 1998 by Juergen Pfeifer                                    --
--                                                                          --
-- Permission is hereby granted, free of charge, to any person obtaining a  --
-- copy of this software and associated documentation files (the            --
-- "Software"), to deal in the Software without restriction, including      --
-- without limitation the rights to use, copy, modify, merge, publish,      --
-- distribute, distribute with modifications, sublicense, and/or sell       --
-- copies of the Software, and to permit persons to whom the Software is    --
-- furnished to do so, subject to the following conditions:                 --
--                                                                          --
-- The above copyright notice and this permission notice shall be included  --
-- in all copies or substantial portions of the Software.                   --
--                                                                          --
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  --
-- OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               --
-- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   --
-- IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   --
-- DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    --
-- OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    --
-- THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               --
--                                                                          --
-- Except as contained in this notice, the name(s) of the above copyright   --
-- holders shall not be used in advertising or otherwise to promote the     --
-- sale, use or other dealings in this Software without prior written       --
-- authorization.                                                           --
------------------------------------------------------------------------------
--  Author: Juergen Pfeifer <Juergen.Pfeifer@T-Online.de> 1998
--  Version Control
--  $Revision: 1.3 $
------------------------------------------------------------------------------
with Interfaces.C_Streams;
with Interfaces.C;

package body GNU.plotutil.Device is

   package CIO renames Interfaces.C_Streams;
   subtype C_Int    is Interfaces.C.int;
   subtype C_String is Interfaces.C.char_array;

   function newpl (Name  : C_String;
                   F_In  : CIO.FILEs;
                   F_Out : CIO.FILEs;
                   F_Err : CIO.FILEs) return Plotter;
   pragma Import (C, newpl, "pl_newpl");

   function New_Plotter (Device_Type, Filename : String) return Plotter is
      Plt : Plotter;
      F   : CIO.FILEs;
      M   : constant C_String := Interfaces.C.To_C ("w");
   begin
      F := CIO.fopen (Interfaces.C.To_C (Filename)'Address,
                      M'Address);
      Plt := newpl (Interfaces.C.To_C (Device_Type),
                    CIO.NULL_Stream,
                    F,
                    CIO.stderr);
      if Plt < 0 then
         raise Plot_Exception;
      else
         return Plt;
      end if;
   end New_Plotter;

   function New_Plotter (Device_Type : String) return Plotter is
      Plt : constant Plotter := newpl (Interfaces.C.To_C (Device_Type),
                                       CIO.NULL_Stream,
                                       CIO.stdout,
                                       CIO.stderr);
   begin
      if Plt < 0 then
         raise Plot_Exception;
      else
         return Plt;
      end if;
   end New_Plotter;

end GNU.plotutil.Device;
