------------------------------------------------------------------------------
--                                                                          --
--                           GNAT libplot binding                           --
--                                                                          --
--                            GNU.plotutil.fplot                            --
--                                                                          --
--                                 B O D Y                                  --
--                                                                          --
------------------------------------------------------------------------------
-- Copyright (c) 1998 by Juergen Pfeifer                                    --
--                                                                          --
-- Permission is hereby granted, free of charge, to any person obtaining a  --
-- copy of this software and associated documentation files (the            --
-- "Software"), to deal in the Software without restriction, including      --
-- without limitation the rights to use, copy, modify, merge, publish,      --
-- distribute, distribute with modifications, sublicense, and/or sell       --
-- copies of the Software, and to permit persons to whom the Software is    --
-- furnished to do so, subject to the following conditions:                 --
--                                                                          --
-- The above copyright notice and this permission notice shall be included  --
-- in all copies or substantial portions of the Software.                   --
--                                                                          --
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  --
-- OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               --
-- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   --
-- IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   --
-- DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    --
-- OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    --
-- THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               --
--                                                                          --
-- Except as contained in this notice, the name(s) of the above copyright   --
-- holders shall not be used in advertising or otherwise to promote the     --
-- sale, use or other dealings in this Software without prior written       --
-- authorization.                                                           --
------------------------------------------------------------------------------
--  Author: Juergen Pfeifer <Juergen.Pfeifer@T-Online.de> 1998
--  Version Control
--  $Revision: 1.3 $
------------------------------------------------------------------------------
with Interfaces.C.Strings;
with Interfaces.C.Pointers;
use  Interfaces.C;
use  Interfaces.C.Strings;

package body GNU.plotutil.fplot is

   subtype C_Int is Interfaces.C.int;

   procedure Space (X0, Y0, X1, Y1 : in Real) is
      function C_space (px0 : double;
                        py0 : double;
                        px1 : double;
                        py1 : double)
                        return C_Int;
      pragma Import (C, C_space, "pl_fspace");

      Res : constant C_Int := C_space (double (X0),
                                       double (Y0),
                                       double (X1),
                                       double (Y1));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Space;

   procedure Space_Extended (X0, Y0, X1, Y1, X2, Y2 : in Real) is
      function C_space2 (px0 : double;
                         py0 : double;
                         px1 : double;
                         py1 : double;
                         px2 : double;
                         py2 : double) return C_Int;
      pragma Import (C, C_space2, "pl_fspace2");

      Res : constant C_Int := C_space2 (double (X0),
                                        double (Y0),
                                        double (X1),
                                        double (Y1),
                                        double (X2),
                                        double (Y2));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Space_Extended;

   procedure Move (X, Y : in Real) is
      function fmove (Px, Py : double) return C_Int;
      pragma Import (C, fmove, "pl_fmove");

      Res : constant C_Int := fmove (double (X), double (Y));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Move;

   procedure Move_Relative (X, Y : in Real) is
      function fmoverel (Px, Py : double) return C_Int;
      pragma Import (C, fmoverel, "pl_fmoverel");

      Res : constant C_Int := fmoverel (double (X), double (Y));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Move_Relative;

   procedure Continue (X, Y : in Real) is
      function fcont (Px, Py : double) return C_Int;
      pragma Import (C, fcont, "pl_fcont");

      Res : constant C_Int := fcont (double (X), double (Y));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Continue;

   procedure Continue_Relative (X, Y : in Real) is
      function fcontrel (Px, Py : double) return C_Int;
      pragma Import (C, fcontrel, "pl_fcontrel");

      Res : constant C_Int := fcontrel (double (X), double (Y));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Continue_Relative;

   procedure Line_Width (Size : in Real'Base) is
      function flinewidth (S : double) return C_Int;
      pragma Import (C, flinewidth, "pl_flinewidth");

      Res : constant C_Int := flinewidth (double (Size));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Line_Width;

   procedure Line_Width is
      function flinewidth (S : double) return C_Int;
      pragma Import (C, flinewidth, "pl_flinewidth");

      Res : constant C_Int := flinewidth (double (-1.0));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Line_Width;

   procedure Line (X0, Y0, X1, Y1 : in Real) is
      function fline (Px0, Py0, Px1, Py1 : double) return C_Int;
      pragma Import (C, fline, "pl_fline");

      Res : constant C_Int := fline (double (X0),
                                     double (Y0),
                                     double (X1),
                                     double (Y1));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Line;

   procedure Line_Relative (X0, Y0, X1, Y1 : in Real) is
      function flinerel (px0, py0, px1, py1 : double) return C_Int;
      pragma Import (C, flinerel, "pl_flinerel");

      Res : constant C_Int := flinerel (double (X0),
                                        double (Y0),
                                        double (X1),
                                        double (Y1));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Line_Relative;

   procedure Point (X, Y : in Real) is
      function fpoint (Px, Py : double) return C_Int;
      pragma Import (C, fpoint, "pl_fpoint");

      Res : constant C_Int := fpoint (double (X), double (Y));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Point;

   procedure Point_Relative (X, Y : in Real) is
      function fpointrel (Px, Py : double) return C_Int;
      pragma Import (C, fpointrel, "pl_fpointrel");

      Res : constant C_Int := fpointrel (double (X), double (Y));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Point_Relative;

   procedure Arc (XC, YC, X0, Y0, X1, Y1 : in Real) is
      function C_Arc (pxc, pyc, px0, py0, px1, py1 : double) return C_Int;
      pragma Import (C, C_Arc, "pl_farc");

      Res : constant C_Int := C_Arc (double (XC),
                                     double (YC),
                                     double (X0),
                                     double (Y0),
                                     double (X1),
                                     double (Y1));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Arc;

   procedure Arc_Relative (XC, YC, X0, Y0, X1, Y1 : in Real) is
      function C_Arcrel (pxc, pyc, px0, py0, px1, py1 : double) return C_Int;
      pragma Import (C, C_Arcrel, "pl_farcrel");

      Res : constant C_Int := C_Arcrel (double (XC),
                                        double (YC),
                                        double (X0),
                                        double (Y0),
                                        double (X1),
                                        double (Y1));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Arc_Relative;

   procedure Elliptical_Arc (XC, YC, X0, Y0, X1, Y1 : in Real) is
      function fellarc (pxc, pyc, px0, py0, px1, py1 : double) return C_Int;
      pragma Import (C, fellarc, "pl_fellarc");

      Res : constant C_Int := fellarc (double (XC),
                                       double (YC),
                                       double (X0),
                                       double (Y0),
                                       double (X1),
                                       double (Y1));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Elliptical_Arc;

   procedure Elliptical_Arc_Relative (XC, YC, X0, Y0, X1, Y1 : in Real) is
      function fellarcrel (pxc, pyc, px0, py0, px1, py1 : double) return C_Int;
      pragma Import (C, fellarcrel, "pl_fellarcrel");

      Res : constant C_Int := fellarcrel (double (XC),
                                          double (YC),
                                          double (X0),
                                          double (Y0),
                                          double (X1),
                                          double (Y1));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Elliptical_Arc_Relative;

   procedure Box (X0, Y0, X1, Y1 : in Real) is
      function C_Box (px0, py0, px1, py1 : double) return C_Int;
      pragma Import (C, C_Box, "pl_fbox");

      Res : constant C_Int := C_Box (double (X0),
                                     double (Y0),
                                     double (X1),
                                     double (Y1));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Box;

   procedure Box_Relative (X0, Y0, X1, Y1 : in Real) is
      function C_Boxrel (px0, py0, px1, py1 : double) return C_Int;
      pragma Import (C, C_Boxrel, "pl_fboxrel");

      Res : constant C_Int := C_Boxrel (double (X0),
                                        double (Y0),
                                        double (X1),
                                        double (Y1));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Box_Relative;

   procedure Circle (XC, YC : in Real; R : in Real'Base) is
      function C_Circle (X0, Y0, Rad : double) return C_Int;
      pragma Import (C, C_Circle, "pl_fcircle");

      Res : constant C_Int := C_Circle (double (XC), double (YC), double (R));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Circle;

   procedure Circle_Relative (XC, YC : in Real; R : in Real'Base) is
      function C_Circlerel (X0, Y0, Rad : double) return C_Int;
      pragma Import (C, C_Circlerel, "pl_fcirclerel");

      Res : constant C_Int := C_Circlerel (double (XC),
                                           double (YC),
                                           double (R));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Circle_Relative;

   procedure Ellipse (XC, YC : in Real; Rx, Ry, Angle : in Real'Base) is
      function fellipse (px, py, prx, pry, pang : double) return C_Int;
      pragma Import (C, fellipse, "pl_fellipse");

      Res : constant C_Int := fellipse (double (XC),
                                        double (YC),
                                        double (Rx),
                                        double (Ry),
                                        double (Angle));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Ellipse;

   procedure Ellipse_Relative (XC, YC : in Real; Rx, Ry, Angle : in Real'Base)
   is
      function fellipserel (px, py, prx, pry, pang : double) return C_Int;
      pragma Import (C, fellipserel, "pl_fellipserel");

      Res : constant C_Int := fellipserel (double (XC),
                                           double (YC),
                                           double (Rx),
                                           double (Ry),
                                           double (Angle));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Ellipse_Relative;

   procedure Quadratic_Bezier_Curve (X0, Y0, X1, Y1, X2, Y2 : in Real) is
      function bezier2 (X0, Y0, X1, Y1, X2, Y2 : double) return C_Int;
      pragma Import (C, bezier2, "pl_fbezier2");

      Res : constant C_Int := bezier2 (double (X0), double (Y0),
                                       double (X1), double (Y1),
                                       double (X2), double (Y2));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Quadratic_Bezier_Curve;

   procedure Quadratic_Bezier_Curve_Relative (X0, Y0,
                                                X1, Y1, X2, Y2 : in Real)
   is
      function bezier2rel (X0, Y0, X1, Y1, X2, Y2 : double) return C_Int;
      pragma Import (C, bezier2rel, "pl_fbezier2rel");

      Res : constant C_Int := bezier2rel (double (X0), double (Y0),
                                          double (X1), double (Y1),
                                          double (X2), double (Y2));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Quadratic_Bezier_Curve_Relative;

   procedure Cubic_Bezier_Curve (X0, Y0, X1, Y1, X2, Y2, X3, Y3 : in Real)
   is
      function bezier3 (X0, Y0, X1, Y1, X2, Y2, X3, Y3 : double) return C_Int;
      pragma Import (C, bezier3, "pl_fbezier3");

      Res : constant C_Int := bezier3 (double (X0), double (Y0),
                                       double (X1), double (Y1),
                                       double (X2), double (Y2),
                                       double (X3), double (Y3));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Cubic_Bezier_Curve;

   procedure Cubic_Bezier_Curve_Relative (X0, Y0,
                                            X1, Y1, X2, Y2, X3, Y3 : in Real)
   is
      function bezier3rel (X0, Y0,
                             X1, Y1, X2, Y2, X3, Y3 : double) return C_Int;
      pragma Import (C, bezier3rel, "pl_fbezier3rel");

      Res : constant C_Int := bezier3rel (double (X0), double (Y0),
                                          double (X1), double (Y1),
                                          double (X2), double (Y2),
                                          double (X3), double (Y3));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Cubic_Bezier_Curve_Relative;

   procedure Marker (X, Y  : in Real;
                     Shape : in Marker_Type;
                     Size  : in Real'Base) is
      function C_Marker (px, py : double;
                         T : C_Int; S : double) return C_Int;
      pragma Import (C, C_Marker, "pl_fmarker");

      Res : constant C_Int := C_Marker (double (X),
                                        double (Y),
                                        C_Int (Marker_Type'Pos (Shape)),
                                        double (Size));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Marker;

   procedure Marker (X, Y : in Real;
                     Ch   : in Character;
                     Size : in Real'Base) is
      function C_Marker (px, py : double;
                         T : C_Int; S : double) return C_Int;
      pragma Import (C, C_Marker, "pl_fmarker");

      Res : constant C_Int := C_Marker (double (X),
                                        double (Y),
                                        C_Int (Character'Pos (Ch)),
                                        double (Size));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Marker;

   procedure Marker_Relative (X, Y  : in Real;
                              Shape : in Marker_Type;
                              Size  : in Real'Base) is
      function C_Markerrel (px, py : double;
                            T : C_Int; S : double) return C_Int;
      pragma Import (C, C_Markerrel, "pl_fmarkerrel");

      Res : constant C_Int := C_Markerrel (double (X),
                                           double (Y),
                                           C_Int (Marker_Type'Pos (Shape)),
                                           double (Size));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Marker_Relative;

   procedure Marker_Relative (X, Y : in Real;
                              Ch   : in Character;
                              Size : in Real'Base) is
      function C_Markerrel (px, py : double;
                            T : C_Int; S : double) return C_Int;
      pragma Import (C, C_Markerrel, "pl_fmarkerrel");

      Res : constant C_Int := C_Markerrel (double (X),
                                           double (Y),
                                           C_Int (Character'Pos (Ch)),
                                           double (Size));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Marker_Relative;

   function Text_Angle (Angle : in Real'Base) return Real'Base is
      function ftextangle (A : double) return double;
      pragma Import (C, ftextangle, "pl_ftextangle");

      R : constant Real'Base := Real'Base (ftextangle (double (Angle)));
   begin
      return R;
   end Text_Angle;

   function Font_Name (Name : in String) return Real'Base is
      function ffontname (S : char_array) return double;
      pragma Import (C, ffontname, "pl_ffontname");

      R : constant Real'Base :=
        Real'Base (ffontname (Interfaces.C.To_C (Name)));
   begin
      return R;
   end Font_Name;

   function Font_Name return Real'Base is
      function ffontname (S : chars_ptr) return double;
      pragma Import (C, ffontname, "pl_ffontname");

      R : constant Real'Base := Real'Base (ffontname (Null_Ptr));
   begin
      return R;
   end Font_Name;

   function Font_Size (Size : in Real'Base) return Real'Base is
      function ffontsize (S : double) return double;
      pragma Import (C, ffontsize, "pl_ffontsize");

      R : constant Real'Base := Real'Base (ffontsize (double (Size)));
   begin
      return R;
   end Font_Size;

   function Font_Size return Real'Base is
      function ffontsize (S : double) return double;
      pragma Import (C, ffontsize, "pl_ffontsize");

      R : constant Real'Base := Real'Base (ffontsize (double (-1.0)));
   begin
      return R;
   end Font_Size;

   function Label_Width (Text : in String) return Real'Base is
      function flabelwidth (S : in char_array) return double;
      pragma Import (C, flabelwidth, "pl_flabelwidth");

      R : constant Real'Base := Real'Base (flabelwidth (To_C (Text)));
   begin
      return R;
   end Label_Width;

   procedure Concatenate_Transformation (M0, M1,
                                           M2, M3,
                                           TX, TY : in Real'Base) is
      function fconcat (pm0, pm1, pm2, pm3, px, py : double) return C_Int;
      pragma Import (C, fconcat, "pl_fconcat");

      Res : constant C_Int := fconcat (double (M0),
                                       double (M1),
                                       double (M2),
                                       double (M3),
                                       double (TX),
                                       double (TY));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Concatenate_Transformation;

   procedure Rotate (Theta : in Real'Base) is
      function frotate (A : double) return C_Int;
      pragma Import (C, frotate, "pl_frotate");

      Res : constant C_Int := frotate (double (Theta));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Rotate;

   procedure Scale (ScaleX, ScaleY : in Real'Base) is
      function fscale (sx, sy : double) return C_Int;
      pragma Import (C, fscale, "pl_fscale");

      Res : constant C_Int := fscale (double (ScaleX), double (ScaleY));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Scale;

   procedure Translate (TX, TY : in Real'Base) is
      function ftranslate (sx, sy : double) return C_Int;
      pragma Import (C, ftranslate, "pl_ftranslate");

      Res : constant C_Int := ftranslate (double (TX), double (TY));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Translate;

   procedure Line_Dash (Dash_Distances : in Dashes;
                        Offset         : in Real'Base) is

      type Double_Array is array (Natural range <>) of aliased double;
      package C_Double_Array is new
        Interfaces.C.Pointers (Natural, double, Double_Array, 0.0);

      function linedash (N      : in C_Int;
                         Dashes : in C_Double_Array.Pointer;
                         Offset : in double) return C_Int;
      pragma Import (C, linedash, "pl_flinedash");

      Res : C_Int;
   begin
      if Dash_Distances'Length = 0 then
         Res := linedash (0, null, double (Offset));
      else
         declare
            N : constant Integer := Dash_Distances'Length;
            D : Double_Array (0 .. (N - 1));
         begin
            for I in Dash_Distances'Range loop
               D (I - Dash_Distances'First) := double (Dash_Distances (I));
            end loop;
            Res := linedash (C_Int (N),
                             D (0)'Unrestricted_Access,
                             double (Offset));
         end;
      end if;
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Line_Dash;

end GNU.plotutil.fplot;
