------------------------------------------------------------------------------
--                                                                          --
--                           GNAT libplot binding                           --
--                                                                          --
--                            GNU.plotutil.iplot                            --
--                                                                          --
--                                 B O D Y                                  --
--                                                                          --
------------------------------------------------------------------------------
-- Copyright (c) 1998 by Juergen Pfeifer                                    --
--                                                                          --
-- Permission is hereby granted, free of charge, to any person obtaining a  --
-- copy of this software and associated documentation files (the            --
-- "Software"), to deal in the Software without restriction, including      --
-- without limitation the rights to use, copy, modify, merge, publish,      --
-- distribute, distribute with modifications, sublicense, and/or sell       --
-- copies of the Software, and to permit persons to whom the Software is    --
-- furnished to do so, subject to the following conditions:                 --
--                                                                          --
-- The above copyright notice and this permission notice shall be included  --
-- in all copies or substantial portions of the Software.                   --
--                                                                          --
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  --
-- OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               --
-- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   --
-- IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   --
-- DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    --
-- OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    --
-- THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               --
--                                                                          --
-- Except as contained in this notice, the name(s) of the above copyright   --
-- holders shall not be used in advertising or otherwise to promote the     --
-- sale, use or other dealings in this Software without prior written       --
-- authorization.                                                           --
------------------------------------------------------------------------------
--  Author: Juergen Pfeifer <Juergen.Pfeifer@T-Online.de> 1998
--  Version Control
--  $Revision: 1.3 $
------------------------------------------------------------------------------
with Interfaces.C.Strings;
with Interfaces.C.Pointers;
use Interfaces.C;
use Interfaces.C.Strings;

package body GNU.plotutil.iplot is

   subtype C_Int is Interfaces.C.int;

   function Num_Check (R : in C_Int) return Int'Base;

   function Num_Check (R : in C_Int) return Int'Base is
   begin
      if R in C_Int (Int'Base'First) .. C_Int (Int'Base'Last) then
         return Int'Base (R);
      else
         raise Plot_Exception;
      end if;
   end Num_Check;
   pragma Inline (Num_Check);

   procedure Space (X0, Y0, X1, Y1 : in Int) is
      function C_space (px0 : C_Int; py0 : C_Int; px1 : C_Int; py1 : C_Int)
        return C_Int;
      pragma Import (C, C_space, "pl_space");

      Res : constant C_Int := C_space (C_Int (X0),
                                       C_Int (Y0),
                                       C_Int (X1),
                                       C_Int (Y1));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Space;

   procedure Space_Extended (X0, Y0, X1, Y1, X2, Y2 : in Int) is
      function C_space2 (px0 : C_Int;
                         py0 : C_Int;
                         px1 : C_Int;
                         py1 : C_Int;
                         px2 : C_Int;
                         py2 : C_Int) return C_Int;
      pragma Import (C, C_space2, "pl_space2");

      Res : constant C_Int := C_space2 (C_Int (X0),
                                        C_Int (Y0),
                                        C_Int (X1),
                                        C_Int (Y1),
                                        C_Int (X2),
                                        C_Int (Y2));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Space_Extended;

   procedure Move (X, Y : in Int) is
      function C_move (Px, Py : C_Int) return C_Int;
      pragma Import (C, C_move, "pl_move");

      Res : constant C_Int := C_move (C_Int (X), C_Int (Y));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Move;

   procedure Move_Relative (X, Y : in Int) is
      function moverel (Px, Py : C_Int) return C_Int;
      pragma Import (C, moverel, "pl_moverel");

      Res : constant C_Int := moverel (C_Int (X), C_Int (Y));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Move_Relative;

   procedure Continue (X, Y : in Int) is
      function cont (Px, Py : C_Int) return C_Int;
      pragma Import (C, cont, "pl_cont");

      Res : constant C_Int := cont (C_Int (X), C_Int (Y));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Continue;

   procedure Continue_Relative (X, Y : in Int) is
      function contrel (Px, Py : C_Int) return C_Int;
      pragma Import (C, contrel, "pl_contrel");

      Res : constant C_Int := contrel (C_Int (X), C_Int (Y));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Continue_Relative;

   procedure Line_Width (Size : in Int'Base) is
      function linewidth (S : C_Int) return C_Int;
      pragma Import (C, linewidth, "pl_linewidth");

      Res : constant C_Int := linewidth (C_Int (Size));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Line_Width;

   procedure Line_Width is
      function linewidth (S : C_Int) return C_Int;
      pragma Import (C, linewidth, "pl_linewidth");

      Res : constant C_Int := linewidth (C_Int (-1));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Line_Width;

   procedure Line (X0, Y0, X1, Y1 : in Int) is
      function C_line (Px0, Py0, Px1, Py1 : C_Int) return C_Int;
      pragma Import (C, C_line, "pl_line");

      Res : constant C_Int := C_line (C_Int (X0),
                                      C_Int (Y0),
                                      C_Int (X1),
                                      C_Int (Y1));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Line;

   procedure Line_Relative (X0, Y0, X1, Y1 : in Int) is
      function linerel (px0, py0, px1, py1 : C_Int) return C_Int;
      pragma Import (C, linerel, "pl_linerel");

      Res : constant C_Int := linerel (C_Int (X0),
                                       C_Int (Y0),
                                       C_Int (X1),
                                       C_Int (Y1));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Line_Relative;

   procedure Point (X, Y : in Int) is
      function point (Px, Py : C_Int) return C_Int;
      pragma Import (C, point, "pl_point");

      Res : constant C_Int := point (C_Int (X), C_Int (Y));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Point;

   procedure Point_Relative (X, Y : in Int) is
      function pointrel (Px, Py : C_Int) return C_Int;
      pragma Import (C, pointrel, "pl_pointrel");

      Res : constant C_Int := pointrel (C_Int (X), C_Int (Y));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Point_Relative;

   procedure Arc (XC, YC, X0, Y0, X1, Y1 : in Int) is
      function C_Arc (pxc, pyc, px0, py0, px1, py1 : C_Int) return C_Int;
      pragma Import (C, C_Arc, "pl_arc");

      Res : constant C_Int := C_Arc (C_Int (XC),
                                     C_Int (YC),
                                     C_Int (X0),
                                     C_Int (Y0),
                                     C_Int (X1),
                                     C_Int (Y1));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Arc;

   procedure Arc_Relative (XC, YC, X0, Y0, X1, Y1 : in Int) is
      function C_Arcrel (pxc, pyc, px0, py0, px1, py1 : C_Int) return C_Int;
      pragma Import (C, C_Arcrel, "pl_arcrel");

      Res : constant C_Int := C_Arcrel (C_Int (XC),
                                        C_Int (YC),
                                        C_Int (X0),
                                        C_Int (Y0),
                                        C_Int (X1),
                                        C_Int (Y1));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Arc_Relative;

   procedure Elliptical_Arc (XC, YC, X0, Y0, X1, Y1 : in Int) is
      function ellarc (pxc, pyc, px0, py0, px1, py1 : C_Int) return C_Int;
      pragma Import (C, ellarc, "pl_ellarc");

      Res : constant C_Int := ellarc (C_Int (XC),
                                      C_Int (YC),
                                      C_Int (X0),
                                      C_Int (Y0),
                                      C_Int (X1),
                                      C_Int (Y1));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Elliptical_Arc;

   procedure Elliptical_Arc_Relative (XC, YC, X0, Y0, X1, Y1 : in Int) is
      function ellarcrel (pxc, pyc, px0, py0, px1, py1 : C_Int) return C_Int;
      pragma Import (C, ellarcrel, "pl_ellarcrel");

      Res : constant C_Int := ellarcrel (C_Int (XC),
                                         C_Int (YC),
                                         C_Int (X0),
                                         C_Int (Y0),
                                         C_Int (X1),
                                         C_Int (Y1));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Elliptical_Arc_Relative;

   procedure Box (X0, Y0, X1, Y1 : in Int) is
      function C_Box (px0, py0, px1, py1 : C_Int) return C_Int;
      pragma Import (C, C_Box, "pl_box");

      Res : constant C_Int := C_Box (C_Int (X0),
                                     C_Int (Y0),
                                     C_Int (X1),
                                     C_Int (Y1));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Box;

   procedure Box_Relative (X0, Y0, X1, Y1 : in Int) is
      function C_Boxrel (px0, py0, px1, py1 : C_Int) return C_Int;
      pragma Import (C, C_Boxrel, "pl_boxrel");

      Res : constant C_Int := C_Boxrel (C_Int (X0),
                                        C_Int (Y0),
                                        C_Int (X1),
                                        C_Int (Y1));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Box_Relative;

   procedure Circle (XC, YC : in Int; R : in Int'Base) is
      function C_Circle (X0, Y0, Rad : C_Int) return C_Int;
      pragma Import (C, C_Circle, "pl_circle");

      Res : constant C_Int := C_Circle (C_Int (XC), C_Int (YC), C_Int (R));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Circle;

   procedure Circle_Relative (XC, YC : in Int; R : in Int'Base) is
      function C_Circlerel (X0, Y0, Rad : C_Int) return C_Int;
      pragma Import (C, C_Circlerel, "pl_circlerel");

      Res : constant C_Int := C_Circlerel (C_Int (XC), C_Int (YC), C_Int (R));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Circle_Relative;

   procedure Ellipse (XC, YC : in Int; Rx, Ry, Angle : in Int'Base) is
      function C_Ellipse (px, py, prx, pry, pang : C_Int) return C_Int;
      pragma Import (C, C_Ellipse, "pl_ellipse");

      Res : constant C_Int := C_Ellipse (C_Int (XC),
                                         C_Int (YC),
                                         C_Int (Rx),
                                         C_Int (Ry),
                                         C_Int (Angle));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Ellipse;

   procedure Ellipse_Relative (XC, YC : in Int; Rx, Ry, Angle : in Int'Base) is
      function C_Ellipserel (px, py, prx, pry, pang : C_Int) return C_Int;
      pragma Import (C, C_Ellipserel, "pl_ellipserel");

      Res : constant C_Int := C_Ellipserel (C_Int (XC),
                                            C_Int (YC),
                                            C_Int (Rx),
                                            C_Int (Ry),
                                            C_Int (Angle));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Ellipse_Relative;

   procedure Quadratic_Bezier_Curve (X0, Y0, X1, Y1, X2, Y2 : in Int) is
      function bezier2 (X0, Y0, X1, Y1, X2, Y2 : C_Int) return C_Int;
      pragma Import (C, bezier2, "pl_bezier2");

      Res : constant C_Int := bezier2 (C_Int (X0), C_Int (Y0),
                                       C_Int (X1), C_Int (Y1),
                                       C_Int (X2), C_Int (Y2));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Quadratic_Bezier_Curve;

   procedure Quadratic_Bezier_Curve_Relative (X0, Y0, X1, Y1, X2, Y2 : in Int)
   is
      function bezier2rel (X0, Y0, X1, Y1, X2, Y2 : C_Int) return C_Int;
      pragma Import (C, bezier2rel, "pl_bezier2rel");

      Res : constant C_Int := bezier2rel (C_Int (X0), C_Int (Y0),
                                          C_Int (X1), C_Int (Y1),
                                          C_Int (X2), C_Int (Y2));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Quadratic_Bezier_Curve_Relative;

   procedure Cubic_Bezier_Curve (X0, Y0, X1, Y1, X2, Y2, X3, Y3 : in Int)
   is
      function bezier3 (X0, Y0, X1, Y1, X2, Y2, X3, Y3 : C_Int) return C_Int;
      pragma Import (C, bezier3, "pl_bezier3");

      Res : constant C_Int := bezier3 (C_Int (X0), C_Int (Y0),
                                       C_Int (X1), C_Int (Y1),
                                       C_Int (X2), C_Int (Y2),
                                       C_Int (X3), C_Int (Y3));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Cubic_Bezier_Curve;

   procedure Cubic_Bezier_Curve_Relative (X0, Y0,
                                            X1, Y1, X2, Y2, X3, Y3 : in Int)
   is
      function bezier3rel (X0, Y0,
                             X1, Y1, X2, Y2, X3, Y3 : C_Int) return C_Int;
      pragma Import (C, bezier3rel, "pl_bezier3rel");

      Res : constant C_Int := bezier3rel (C_Int (X0), C_Int (Y0),
                                          C_Int (X1), C_Int (Y1),
                                          C_Int (X2), C_Int (Y2),
                                          C_Int (X3), C_Int (Y3));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Cubic_Bezier_Curve_Relative;

   procedure Marker (X, Y  : in Int;
                     Shape : in Marker_Type;
                     Size  : in Int'Base) is
      function C_Marker (px, py, T, S : C_Int) return C_Int;
      pragma Import (C, C_Marker, "pl_marker");

      Res : constant C_Int := C_Marker (C_Int (X),
                                        C_Int (Y),
                                        C_Int (Marker_Type'Pos (Shape)),
                                        C_Int (Size));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Marker;

   procedure Marker (X, Y : in Int;
                     Ch   : in Character;
                     Size : in Int'Base) is
      function C_Marker (px, py, T, S : C_Int) return C_Int;
      pragma Import (C, C_Marker, "pl_marker");

      Res : constant C_Int := C_Marker (C_Int (X),
                                        C_Int (Y),
                                        C_Int (Character'Pos (Ch)),
                                        C_Int (Size));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Marker;

   procedure Marker_Relative (X, Y  : in Int;
                              Shape : in Marker_Type;
                              Size  : in Int'Base) is
      function C_Markerrel (px, py, T, S : C_Int) return C_Int;
      pragma Import (C, C_Markerrel, "pl_markerrel");

      Res : constant C_Int := C_Markerrel (C_Int (X),
                                           C_Int (Y),
                                           C_Int (Marker_Type'Pos (Shape)),
                                           C_Int (Size));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Marker_Relative;

   procedure Marker_Relative (X, Y : in Int;
                              Ch   : in Character;
                              Size : in Int'Base) is
      function C_Markerrel (px, py, T, S : C_Int) return C_Int;
      pragma Import (C, C_Markerrel, "pl_markerrel");

      Res : constant C_Int := C_Markerrel (C_Int (X),
                                           C_Int (Y),
                                           C_Int (Character'Pos (Ch)),
                                           C_Int (Size));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Marker_Relative;

   function Text_Angle (Angle : in Int'Base) return Int'Base is
      function textangle (A : C_Int) return C_Int;
      pragma Import (C, textangle, "pl_textangle");

      R : constant C_Int := textangle (C_Int (Angle));
   begin
      return Num_Check (R);
   end Text_Angle;

   function Font_Name (Name : in String) return Int'Base is
      function fontname (S : char_array) return C_Int;
      pragma Import (C, fontname, "pl_fontname");

      R : constant C_Int := fontname (Interfaces.C.To_C (Name));
   begin
      return Num_Check (R);
   end Font_Name;

   function Font_Name return Int'Base is
      function fontname (S : chars_ptr) return C_Int;
      pragma Import (C, fontname, "pl_fontname");

      R : constant C_Int := fontname (Null_Ptr);
   begin
      return Num_Check (R);
   end Font_Name;

   function Font_Size (Size : in Int'Base) return Int'Base is
      function fontsize (S : C_Int) return C_Int;
      pragma Import (C, fontsize, "pl_fontsize");

      R : constant C_Int := fontsize (C_Int (Size));
   begin
      return Num_Check (R);
   end Font_Size;

   function Font_Size return Int'Base is
      function fontsize (S : C_Int) return C_Int;
      pragma Import (C, fontsize, "pl_fontsize");

      R : constant C_Int := fontsize (C_Int (-1));
   begin
      return Num_Check (R);
   end Font_Size;

   function Label_Width (Text : in String) return Int'Base is
      function labelwidth (S : in char_array) return C_Int;
      pragma Import (C, labelwidth, "pl_labelwidth");

      R : constant C_Int := labelwidth (To_C (Text));
   begin
      return Num_Check (R);
   end Label_Width;


   procedure Line_Dash (Dash_Distances : in Dashes;
                        Offset         : in Int'Base) is

      type Int_Array is array (Natural range <>) of aliased C_Int;
      package C_Int_Array is new
        Interfaces.C.Pointers (Natural, C_Int, Int_Array, 0);

      function linedash (N      : in C_Int;
                         Dashes : in C_Int_Array.Pointer;
                         Offset : in C_Int) return C_Int;
      pragma Import (C, linedash, "pl_linedash");

      Res : C_Int;
   begin
      if Dash_Distances'Length = 0 then
         Res := linedash (0, null, C_Int (Offset));
      else
         declare
            N : constant Integer := Dash_Distances'Length;
            D : Int_Array (0 .. (N - 1));
         begin
            for I in Dash_Distances'Range loop
               D (I - Dash_Distances'First) := C_Int (Dash_Distances (I));
            end loop;
            Res := linedash (C_Int (N),
                             D (0)'Unrestricted_Access,
                             C_Int (Offset));
         end;
      end if;
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Line_Dash;


end GNU.plotutil.iplot;
