------------------------------------------------------------------------------
--                                                                          --
--                           GNAT libplot binding                           --
--                                                                          --
--                              GNU.plotutil                                --
--                                                                          --
--                                 B O D Y                                  --
--                                                                          --
------------------------------------------------------------------------------
-- Copyright (c) 1998 by Juergen Pfeifer                                    --
--                                                                          --
-- Permission is hereby granted, free of charge, to any person obtaining a  --
-- copy of this software and associated documentation files (the            --
-- "Software"), to deal in the Software without restriction, including      --
-- without limitation the rights to use, copy, modify, merge, publish,      --
-- distribute, distribute with modifications, sublicense, and/or sell       --
-- copies of the Software, and to permit persons to whom the Software is    --
-- furnished to do so, subject to the following conditions:                 --
--                                                                          --
-- The above copyright notice and this permission notice shall be included  --
-- in all copies or substantial portions of the Software.                   --
--                                                                          --
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  --
-- OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               --
-- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   --
-- IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   --
-- DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    --
-- OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    --
-- THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               --
--                                                                          --
-- Except as contained in this notice, the name(s) of the above copyright   --
-- holders shall not be used in advertising or otherwise to promote the     --
-- sale, use or other dealings in this Software without prior written       --
-- authorization.                                                           --
------------------------------------------------------------------------------
--  Author: Juergen Pfeifer <Juergen.Pfeifer@T-Online.de> 1998
--  Version Control
--  $Revision: 1.3 $
------------------------------------------------------------------------------
with Ada.Characters.Handling; use Ada.Characters.Handling;
with Ada.Exceptions;
with Interfaces.C.Strings;

package body GNU.plotutil is

   subtype C_Int    is Interfaces.C.int;
   subtype C_String is Interfaces.C.char_array;
   subtype P_String is Interfaces.C.Strings.chars_ptr;
   use type C_Int;

   Current_Plotter : Plotter := Default_Plotter;

   procedure Select_Plotter (P : in Plotter := Default_Plotter) is
      function selectpl (H : in Plotter) return C_Int;
      pragma Import (C, selectpl, "pl_selectpl");

      Res : constant C_Int := selectpl (P);
   begin
      if Res < 0 then
         raise Plot_Exception;
      else
         Current_Plotter := P;
      end if;
   end Select_Plotter;

   procedure Delete (P : in Plotter) is
      function deletepl (H : in Plotter) return C_Int;
      pragma Import (C, deletepl, "pl_deletepl");

      Res : C_Int;
      PO  : constant Plotter := Current_Plotter;
   begin
      if P = Current_Plotter then
         Select_Plotter;
         Res := deletepl (PO);
      else
         Res := deletepl (P);
      end if;
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Delete;

   procedure Delete is
   begin
      Delete (Current_Plotter);
   end Delete;

      procedure Open is
      function openpl return C_Int;
      pragma Import (C, openpl, "pl_openpl");

      Res : constant C_Int := openpl;
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Open;

   procedure Close is
      function closepl return C_Int;
      pragma Import (C, closepl, "pl_closepl");

      Res : constant C_Int := closepl;
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Close;

   procedure Erase is
      function c_erase return C_Int;
      pragma Import (C, c_erase, "pl_erase");

      Res : constant C_Int := c_erase;
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Erase;

   procedure Flush is
      function flushpl return C_Int;
      pragma Import (C, flushpl, "pl_flushpl");

      Res : constant C_Int := flushpl;
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Flush;
   --
   procedure Set_Parameter (Parameter, Value : in String) is
      function parampl (P, V : C_String) return C_Int;
      pragma Import (C, parampl, "pl_parampl");
      Res : constant C_Int := parampl (Interfaces.C.To_C (Parameter),
                                       Interfaces.C.To_C (Value));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Set_Parameter;

   procedure Set_Parameter (Parameter : in String; Value : in System.Address)
   is
      function parampl (P : C_String; V : System.Address) return C_Int;
      pragma Import (C, parampl, "pl_parampl");

      Res : constant C_Int := parampl (Interfaces.C.To_C (Parameter),
                                       Value);
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Set_Parameter;

   --  ######################################################################
   --
   procedure ColorName (Name : in String) is
      function colorname (S : C_String) return C_Int;
      pragma Import (C, colorname, "pl_colorname");

      Res : constant C_Int := colorname (Interfaces.C.To_C (Name));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end ColorName;

   procedure Color (Red, Green, Blue : in RGB_Value) is
      function color (R, G, B : C_Int) return C_Int;
      pragma Import (C, color, "pl_color");

      Res : constant C_Int := color (C_Int (Red),
                                     C_Int (Green),
                                     C_Int (Blue));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Color;
   --
   --  ######################################################################
   --
   procedure Fill_ColorName (Name : in String) is
      function fillcolorname (S : C_String) return C_Int;
      pragma Import (C, fillcolorname, "pl_fillcolorname");

      Res : constant C_Int := fillcolorname (Interfaces.C.To_C (Name));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Fill_ColorName;

   procedure Fill_Color (Red, Green, Blue : in RGB_Value) is
      function fillcolor (R, G, B : C_Int) return C_Int;
      pragma Import (C, fillcolor, "pl_fillcolor");

      Res : constant C_Int := fillcolor (C_Int (Red),
                                         C_Int (Green),
                                         C_Int (Blue));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Fill_Color;
   --
   --  ######################################################################
   --
   procedure Pen_ColorName (Name : in String) is
      function pencolorname (S : C_String) return C_Int;
      pragma Import (C, pencolorname, "pl_pencolorname");

      Res : constant C_Int := pencolorname (Interfaces.C.To_C (Name));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Pen_ColorName;

   procedure Pen_Color (Red, Green, Blue : in RGB_Value) is
      function pencolor (R, G, B : C_Int) return C_Int;
      pragma Import (C, pencolor, "pl_pencolor");

      Res : constant C_Int := pencolor (C_Int (Red),
                                        C_Int (Green),
                                        C_Int (Blue));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Pen_Color;

   procedure Background_ColorName (Name : in String) is
      function bgcolorname (S : C_String) return C_Int;
      pragma Import (C, bgcolorname, "pl_bgcolorname");

      Res : constant C_Int := bgcolorname (Interfaces.C.To_C (Name));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Background_ColorName;

   procedure Background_Color (Red, Green, Blue : in RGB_Value) is
      function bgcolor (R, G, B : C_Int) return C_Int;
      pragma Import (C, bgcolor, "pl_bgcolor");

      Res : constant C_Int := bgcolor (C_Int (Red),
                                       C_Int (Green),
                                       C_Int (Blue));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Background_Color;
   --
   --  ######################################################################
   --
   type H_Char is array (Horizontal_Justification) of Character;
   type V_Char is array (Vertical_Justification)   of Character;

   H_Map : constant H_Char := (Left   => 'l',
                               Center => 'c',
                               Right  => 'r');
   V_Map : constant V_Char := (Bottom   => 'b',
                               Baseline => 'x',
                               Center   => 'c',
                               Top      => 't');

   procedure Label (H_Justify : in Horizontal_Justification := Left;
                    V_Justify : in Vertical_Justification   := Baseline;
                    Text      : in String) is
      function alabel (H, V : C_Int; L : C_String) return C_Int;
      pragma Import (C, alabel, "pl_alabel");

      H    : constant C_Int := Character'Pos (H_Map (H_Justify));
      V    : constant C_Int := Character'Pos (V_Map (V_Justify));
      Res  : constant C_Int := alabel (H, V, Interfaces.C.To_C (Text));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Label;
   --
   --  ######################################################################
   --
   procedure Line_Mode (Mode : in Line_Type) is
      function linemod (S : C_String) return C_Int;
      pragma Import (C, linemod, "pl_linemod");

      Res : constant C_Int :=
        linemod (Interfaces.C.To_C (To_Lower (Line_Type'Image (Mode))));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Line_Mode;
   --
   --  ######################################################################
   --
   procedure Cap_Mode (Style : in Cap_Style := Butt) is
      function capmod (S : C_String) return C_Int;
      pragma Import (C, capmod, "pl_capmod");

      Res : constant C_Int :=
        capmod (Interfaces.C.To_C (To_Lower (Cap_Style'Image (Style))));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Cap_Mode;
   --
   --  ######################################################################
   --
   procedure Filltype (Level : in Filltype_Level := 0) is
      function C_Filltype (L : C_Int) return C_Int;
      pragma Import (C, C_Filltype, "pl_filltype");

      Res : constant C_Int := C_Filltype (C_Int (Level));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Filltype;
   --
   --  ######################################################################
   --
   procedure End_Path is
      function endpath return C_Int;
      pragma Import (C, endpath, "pl_endpath");

      Res : constant C_Int := endpath;
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end End_Path;
   --
   --  ######################################################################
   --
   function Has_Capability (Capability : String) return Capability_Existence is
      function havecap (S : C_String) return C_Int;
      pragma Import (C, havecap, "pl_havecap");

      R : constant C_Int := havecap (Interfaces.C.To_C (Capability));
   begin
      case R is
         when 0 => return No;
         when 1 => return Yes;
         when 2 => return Maybe;
         when others => raise Plot_Exception;
      end case;
   end Has_Capability;
   --
   --  ######################################################################
   --
   procedure Join_Mode (Style : in Join_Style := Miter) is
      function joinmod (S : C_String) return C_Int;
      pragma Import (C, joinmod, "pl_joinmod");

      Res : constant C_Int :=
        joinmod (Interfaces.C.To_C (To_Lower (Join_Style'Image (Style))));
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Join_Mode;
   --
   procedure Miter_Limit (Limit : in Interfaces.C.double) is
      function fmiterlimit (Limit : in Interfaces.C.double) return C_Int;
      pragma Import (C, fmiterlimit, "pl_fmiterlimit");

      Res : constant C_Int := fmiterlimit (Limit);
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Miter_Limit;

   --  ######################################################################
   --
   procedure Pop_State is
      function restorestate return C_Int;
      pragma Import (C, restorestate, "pl_restorestate");

      Res : constant C_Int := restorestate;
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Pop_State;

   procedure Push_State is
      function savestate return C_Int;
      pragma Import (C, savestate, "pl_savestate");

      Res : constant C_Int := savestate;
   begin
      if Res < 0 then
         raise Plot_Exception;
      end if;
   end Push_State;

   type Error_Handler is access procedure (Err : in P_String);
   pragma Convention (C, Error_Handler);

   Error_Hook : Error_Handler;
   pragma Import (C, Error_Hook, "libplot_error_handler");

   procedure Ada_Error_Handler (Err : in P_String);
   pragma Convention (C, Ada_Error_Handler);

   procedure Ada_Error_Handler (Err : in P_String) is
      Message : constant String := Interfaces.C.Strings.Value (Err);
   begin
      Ada.Exceptions.Raise_Exception (Plot_Exception'Identity, Message);
   end Ada_Error_Handler;

begin
   Error_Hook := Ada_Error_Handler'Access;
end GNU.plotutil;
